﻿--[[
Name: LibGuildBankComm-1.0
Revision: $Revision: 1 $
Author(s): Myrroddin (psvander@gmail.com)
Documentation: http://www.wowace.com/index.php/LibResComm-1.0
SVN:  svn://svn.wowace.com/wow/librescomm-1-0/mainline/trunk
Description: Saves guild bank contents and sends/receives updates over the addon channel
Dependencies: LibStub, CallbackHandler-1.0, ChatThrottleLib
]]

local MAJOR_VERSION = "LibGuildBankComm-1.0"
local MINOR_VERSION = 90000 + tonumber(("$Revision: 1 $"):match("%d+"))

local lib = LibStub:NewLibrary(MAJOR_VERSION, MINOR_VERSION)
if not lib then return end

-- Older version loaded. Clean up
if lib.Stop then
	lib.Stop()
end

--------------------------------------------------------------------------------
-- Start library
--
lib.Stop = function()
	lib.frame:UnregisterAllEvents()
end
lib:Start()

lib.guildBank = lib.guildBank or {}

if not lib.frame then
  lib.frame = CreateFrame("Frame")
end

lib.frame:SetScript("OnEvent", function(this, event, ...)
	this[event](this, ...)
end)
lib.frame:UnregisterAllEvents()

if not lib.Callbacks then
	lib.Callbacks = LibStub("CallbackHandler-1.0"):New(lib)
end

local function OnUpdate()
  -- timing stuff here
end

lib.frame:SetScript("OnUpdate", OnUpdate);

local guilded = IsInGuild("player")
local guildName = GetGuildInfo("player")
local pName = UnitName("player")
local localTimeStamp

local function commSend(contents, distribution, target)
	ChatThrottleLib:SendAddonMessage("gBankComm", contents, distribution, target)
end

function lib:Start()
	lib.frame:RegisterEvent("PLAYER_ENTERING_WORLD")
	lib.frame:RegisterEvent("CHAT_MSG_ADDON")
	lib.frame:RegisterEvent("PLAYER_GUILD_UPDATE")
	if guilded == 1 then
		lib.frame:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
		lib.frame:RegisterEvent("GUILDBANKFRAME_OPENED")
		lib.frame:RegisterEvent("GUILDBANKFRAME_CLOSED")
		lib.frame:RegisterEvent("GUILDBANK_UPDATE_MONEY")
		lib.frame:RegisterEvent("GUILDBANK_UPDATE_TABS")
	end
end

function lib.frame:CHAT_MSG_ADDON(prefix, msg, distribution, sender)
	if prefix ~= "gBankComm" then return end
	if sender == "player" then return end
	
	local gbContents = lib.guildBank
	
	for cmd in msg:gmatch("(%a+)%s?([^#]*)") do
		if cmd == "REQEST" then -- someone logged in and requests data
			if localTimeStamp ~= nil then
				commSend("gBankComm", localTimeStamp, sender) -- so we send them out time stamp, so they compare to find out whom to request the data from
			end
		elseif cmd == "SYNC" then -- someone specifically wants our data, as it is the newest
			commSend("gBankCOmm", gbContents, sender)
		elseif cmd == "UPDATE" then -- someone just visited the guild bank and has recent data
			gbContents = msg			
			localTimeStamp = gbContents[guildName].timeStamp
			lib.CallBacks:Fire("GuildBankComm_ContentsChanged", sender, gbContents)
		end
	end
end

function lib:PLAYER_ENTERING_WORLD()
	commSend("gBankComm", "REQUEST", "GUILD")
end

function lib:PLAYER_GUILD_UPDATE()
	if guilded ~= 1 then
		wipe(lib.guildBank)
		lib.frame:UnregisterEvent("GUILDBANKBAGSLOTS_CHANGED")		
		lib.frame:UnregisterEvent("GUILDBANKFRAME_CLOSED")
		lib.frame:UnregisterEvent("GUILDBANK_UPDATE_MONEY")
		lib.frame:UnregisterEvent("GUILDBANK_UPDATE_TABS")
	else
		lib.frame:RegisterEvent("GUILDBANKBAGSLOTS_CHANGED")
		lib.frame:RegisterEvent("GUILDBANKFRAME_OPENED")
		lib.frame:RegisterEvent("GUILDBANKFRAME_CLOSED")
		lib.frame:RegisterEvent("GUILDBANK_UPDATE_MONEY")
		lib.frame:RegisterEvent("GUILDBANK_UPDATE_TABS")
	end
	guildName = GetGuildInfo("player")
	guilded = IsInGuild("player")
end

function lib:GUILDBANK_UPDATE_MONEY(unit)
	if unit ~= "player" then return end
	lib.guildBank[guildName].Money = GetGuildBankMoney()
end

function lib:GUILDBANKFRAME_OPENED()
	lib.guildBank[guildName].numTabs = GetNumGuildBankTabs()
	lib.guildBank[guildName].Money = GetGuildBankMoney()
	
	local name, icon, view	
	for page = 1, lib.guildBank[guildName].numTabs do
		name, icon, view = GetGuildBankTabInfo(page)
		lib.guildBank[guildName].tab.texture[page] = icon
		lib.guildBank[guildName].tab.name[page] = name
		lib.guildBank[guildName].tab.view[page] = view
	end
end

function lib:GUILDBANKFRAME_CLOSED(unit)
	if unit ~= "player" then return end
	local gBankContents = tostring(lib.guildBank)
	commSend(string.format("gBankComm", "UPDATE %s", gBankContents), "GUILD")
	lib.CallBacks:Fire("GuildBankComm_ContentsChanged", unit, gbContents)
end

function lib:GUILDBANK_UPDATE_TABS()
	lib.guildBank[guildName].numTabs = GetNumGuildBankTabs()
end

function lib:GUILDBANKBAGSLOTS_CHANGED()
	if (GuildBankFrame and GuildBankFrame:IsVisible()) or (BagnonFrameguildbank and BagnonFrameguildbank:IsVisible()) then
		local page = GetCurrentGuildBankTab()
		--98 slots on a page
		for slot = 1, 98 do
			--see if the slot has an item in it
			if GetGuildBankItemLink(page, slot) then
				--if there is link, parse the ID and place that in table w/ key = slot
				_, _, itemString = string.find(GetGuildBankItemLink(page, slot), "^|c%x+|H(.+)|h%[.+%]")
				lib.guildBank[guildName].links[page][slot] = itemString
				
				--get item info too, need for stack sizes in image
				_, num = GetGuildBankItemInfo(page, slot)
				lib.guildBank[guildName].info[page][slot] = num
			else
				--if there is no link, slot is empty... fill slot w/ empty string
				--needed to keep consistent tables with keys from 1 to 98
				lib.guildBank[guildName].links[page][slot] = "empty"
				lib.guildBank[guildName].info[page][slot] = "empty"
			end
			num, itemString = nil, nil
		end
		
		--save a timestamp
		local date = GetDate()
		lib.guildBank[guildName].lastScan = string.format("%s %s %s", date, L["at"], self:TimeStamp())
	end
end

function lib:TimeStamp()
	local hour, minute = GetGameTime()
	if minute < 10 then
		minute = "0"..minute
	end
	
	if hour == 0 then
		return string.format(" %s:%s %s", "12", minute, L["AM"])
	elseif hour == 12 then
		return string.format(" %s:%s %s", hour, minute, L["PM"])
	elseif hour > 12 then
		return string.format(" %s:%s %s", (hour-12), minute, L["PM"])
	else
		return string.format(" %s:%s %s", hour, minute, L["AM"])
	end
end